import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://bincollections.redditchbc.gov.uk/BinCollections/Details"

        data = {"UPRN": user_uprn}

        # Make the GET request
        response = requests.post(URI, data=data)

        # Parse the HTML
        soup = BeautifulSoup(response.content, "html.parser")

        # Find all collection containers
        collection_containers = soup.find_all("div", class_="collection-container")

        # Parse each collection container
        for container in collection_containers:
            # Extract bin type (from heading or image alt attribute)
            bin_type = container.find("img")["alt"]

            # Extract the next collection date (from the caption paragraph)
            next_collection = (
                container.find("p", class_="caption")
                .text.replace("Next collection ", "")
                .strip()
            )

            # Extract additional future collection dates (from the list items)
            future_dates = [li.text.strip() for li in container.find_all("li")]

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    next_collection,
                    "%A, %d %B %Y",
                ).strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

            for date in future_dates:  # Add to the schedule
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(
                        date,
                        "%A, %d %B %Y",
                    ).strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://my.maidstone.gov.uk/service/Find-your-bin-day"
            bin_data = {"bins": []}
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )
            driver.switch_to.frame(iframe_presense)

            wait = WebDriverWait(driver, 60)

            # Postal code input
            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.NAME, "postcode"))
            )
            inputElement_postcodesearch.send_keys(user_postcode)

            time.sleep(5)

            inputElement_postcodesearch.send_keys(Keys.TAB + Keys.DOWN)

            dropdown = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, f"//div[contains(text(), ' {user_paon}')]")
                )
            )
            dropdown.click()

            # This website is horrible!
            WebDriverWait(driver, 20).until(
                EC.text_to_be_present_in_element(
                    (By.CSS_SELECTOR, "div.col-collection-panel"), "Next collection"
                )
            )

            # Even then it can still be adding data to the page...
            time.sleep(5)

            # Scraping via Selenium rather than BeautifulSoup, to ensure eveything's loaded
            collection_panels = driver.find_elements(
                By.CSS_SELECTOR, "div.col-collection-panel"
            )

            for panel in collection_panels:
                try:
                    # Get bin type (e.g., General waste, Food waste)
                    bin_type = panel.find_element(
                        By.CSS_SELECTOR, "h3.collectionDataHeader"
                    ).text.strip()
                    # Get next collection date
                    lines = panel.find_elements(By.CSS_SELECTOR, "ul li")
                    for line in lines:
                        if "Next collection" in line.text:
                            date_str = (
                                line.text.split("Next collection")[1]
                                .strip(": ")
                                .strip()
                            )
                            bin_date = datetime.strptime(date_str, "%d/%m/%Y")
                            bin_data["bins"].append(
                                {
                                    "type": bin_type,
                                    "collectionDate": bin_date.strftime(date_format),
                                }
                            )
                except Exception as inner_e:
                    print(f"Skipping one panel due to error: {inner_e}")

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bin_data



from bs4 import BeautifulSoup
import urllib.parse

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        collections = []
        selected_collections = kwargs.get("paon").split(",")
        calendar_urls = []
        run_date = datetime.now().date()

        # For each collection, check if there's a number. Garden bins have no numbers, so we can generate the needed
        # URLs this way
        for item in selected_collections:
            item = item.strip().lower().replace(" ", "_")
            if has_numbers(item):
                calendar_urls.append(
                    f"https://www.gbcbincalendars.co.uk/json/gedling_borough_council_{item}_bin_schedule.json"
                )
            else:
                calendar_urls.append(
                    f"https://www.gbcbincalendars.co.uk/json/gedling_borough_council_{item}_garden_bin_schedule.json"
                )

        # Parse each URL and load future data
        for url in calendar_urls:
            response = requests.get(url)
            if response.status_code != 200:
                raise ConnectionError(f"Could not get response from: {url}")
            json_data = response.json()["collectionDates"]
            for col in json_data:
                bin_date = datetime.strptime(
                    col.get("collectionDate"), "%Y-%m-%d"
                ).date()
                if bin_date >= run_date:
                    collections.append((col.get("alternativeName"), bin_date))

        # Sort the data
        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for bin in ordered_data:
            dict_data = {
                "type": bin[0],
                "collectionDate": bin[1].strftime(date_format),
            }
            data["bins"].append(dict_data)
        print()

        return data
